<?php
namespace Ig\IgRuckzuckevent\Controller;

use Ig\IgPayment\Services\BraintreeService;
use TYPO3\CMS\Core\Utility\GeneralUtility;
use TYPO3\CMS\Core\Utility\ExtensionManagementUtility;
use TYPO3\CMS\Extbase\Utility\DebuggerUtility;
use Ig\IgRuckzuckevent\Utility\UserUtility;
use TYPO3\CMS\Fluid\View\StandaloneView;

/**
 * The EventController handles all actions directly related to Events.
 */
class SubscriptionController extends \TYPO3\CMS\Extbase\Mvc\Controller\ActionController
{
    /**
     * subscriptionRepository
     *
     * @var \Ig\IgRuckzuckevent\Domain\Repository\SubscriptionRepository
     */
    protected $subscriptionRepository = null;

    /**
     * feUserRepository
     *
     * @var \Ig\IgRuckzuckevent\Domain\Repository\FeUserRepository
     */
    protected $feUserRepository = null;

    /**
     * @param \Ig\IgRuckzuckevent\Domain\Repository\SubscriptionRepository $subscriptionRepository
     */
    public function injectSubscriptionRepository(\Ig\IgRuckzuckevent\Domain\Repository\SubscriptionRepository $subscriptionRepository)
    {
        $this->subscriptionRepository = $subscriptionRepository;
    }

    /**
     * @param \Ig\IgRuckzuckevent\Domain\Repository\FeUserRepository $feUserRepository
     */
    public function injectFeUserRepository(\Ig\IgRuckzuckevent\Domain\Repository\FeUserRepository $feUserRepository)
    {
        $this->feUserRepository = $feUserRepository;
    }


    /**
     * Initialize Braintree API
     *
     * @return void
     */
    public function initializeAction()
    {
        $this->braintreeService= GeneralUtility::makeInstance(BraintreeService::class);
    }

    public function listAction()
    {
        if (!UserUtility::isAdminLoggedIn()) {
            exit(0);
        }
        /*
        if ( $this->request->hasArgument('search')) {
    $search = $this->request->getArgument('search');
        } else {
    $search = [];
        }
        if ( $this->request->hasArgument('user')) {
    $user = $this->request->getArgument('user');
        } else {
    $user = null;
        }
        $this->view->assign('search', $search);
        */
        //$users = $this->feUserRepository->findByUsergroup(4);
        $subscription = $this->subscriptionRepository->findAll();
        $this->view->assign('subscriptions', $subscription);
    }



    /**
     * Generates client token and shows payment form for product
     *
     * @param \Ig\IgRuckzuckevent\Domain\Model\SubscriptionOffer $product
     * @param string $referer
     * @param string $startDate
     *
     * @return void
     */
    public function paymentAction(\Ig\IgRuckzuckevent\Domain\Model\SubscriptionOffer $product, $referer, $startDate)
    {
        // Paypal payment
        if ($GLOBALS['TSFE']->loginUser) {
            $this->view->assign('clientToken', $this->braintreeService->generate());
            $this->view->assign('product', $product);
            $this->view->assign('referer', $referer);
            $this->view->assign('startDate', $startDate);
        }
    }

    /**
     * Performs the transaction if the amount coming per post is the same as the price of the product
     *
     * @param \Ig\IgRuckzuckevent\Domain\Model\SubscriptionOffer $product
     * @param string $nonce
     * @param string $startDate
     *
     * @return void
     */
    public function checkoutAction(\Ig\IgRuckzuckevent\Domain\Model\SubscriptionOffer $product, $referer, $startDate)
    {
        $amount = GeneralUtility::_POST('amount');
        $nonce = GeneralUtility::_POST('payment_method_nonce');

        if ($GLOBALS['TSFE']->loginUser && $amount == $product->getPrice()) {
            //if($this->braintreeService->sale($product->getPrice(), $amount, $nonce) === true) {
            $result = $this->braintreeService->sale([
                'amount' => $amount,
                'paymentMethodNonce' => $nonce,
                'options' => [
                    'submitForSettlement' => true
                ]
            ]);
            $user = $this->feUserRepository->findByUid($GLOBALS['TSFE']->fe_user->user['uid']);
            if ($this->braintreeService->isResultSuccess($result)) {
                $transaction = $result->transaction;
                $subscription = GeneralUtility::makeInstance('Ig\\IgRuckzuckevent\\Domain\\Model\\Subscription');
                //      var_dump($result);exit(0);
                $subscription->setSubscriptionOffer($product);
                $subscription->setTransactionId($transaction->id);
                $subscription->setTransactionAmount($transaction->amount);
                $subscription->setTransactionCurrency($transaction->currencyIsoCode);
                $subscription->setPaymentMethod($transaction->paymentInstrumentType);
                $subscription->setFeUser($user);

                $startDateObject = new \DateTime();
                $d = explode('.', $startDate);
                $startDateObject->setDate($d[2], $d[1], $d[0]);

                $inThreeMonths = clone $startDateObject;
                $inThreeMonths->modify('+' . $product->getDuration() . ' month');

                $subscription->setStartDate($startDateObject);
                $subscription->setExpirationDate($inThreeMonths);
                $subscription->setDaysLeft($product->getDays());

                $user->addSubscription($subscription);

                $this->subscriptionRepository->add($subscription);
                $persistenceManager = GeneralUtility::makeInstance('TYPO3\\CMS\\Extbase\\Persistence\\Generic\\PersistenceManager');
                $persistenceManager->persistAll();

                $mailView = $this -> createFluidTemplate('Subscription/Mail.html');
                $mailView -> assign('subscription', $subscription);

                // Create mail object and send mail to email address the user who registered to the event entered
                $mail = GeneralUtility::makeInstance('TYPO3\\CMS\\Core\\Mail\\MailMessage');
                $mail -> setSubject('Physio-Team Naderi: Ihr neues Abonnement')
                    -> setFrom([$this->settings['debugMail'] ? : $this -> settings['mailFrom']])
                    -> setTo([$this->settings['debugMail'] ? : $user -> getEmail()])
                    -> setBody($mailView -> render(), 'text/html')
                    -> send();

                // Same for the admin mail
                $adminMailView = $this -> createFluidTemplate('Subscription/AdminMail.html');
                $adminMailView -> assign('subscription', $subscription);

                if($this->settings['debugMail']) {
                    $adminMailTo = $this->settings['debugMail'];
                } else {
                    $adminMailTo = $this->settings['mailTo'] ? : $this -> settings['mailFrom'];
                }
                $adminMail = GeneralUtility::makeInstance('TYPO3\\CMS\\Core\\Mail\\MailMessage');
                $adminMail -> setSubject('Physio-Team Naderi: Neues Abonnement wurde gelöst')
                    -> setFrom([$this->settings['debugMail'] ? : $this -> settings['mailFrom']])
                    -> setTo([$adminMailTo])
                    -> setBody($adminMailView -> render(), 'text/html')
                    -> send();
            }
            $this->redirect(
                'transaction',
                'Subscription',
                'IgRuckzuckevent',
                [
                    'product' => $product,
                    'referer' => $referer,
                    'transactionId' => $transaction->id,
                    'startDate' => $startDate
                ]
            );
            exit();
        } else {
            $errorString = '';

            foreach ($result->errors->deepAll() as $error) {
                $errorString .= 'Error: ' . $error->code . ': ' . $error->message . "\n";
            }

            $_SESSION['errors'] = $errorString;
        }
        $this->redirect('payment', 'Subscription', 'IgRuckzuckevent', ['product' => $product, 'referer' => $referer, 'startDate' => $startDate]);
        exit();
    }

    /**
     * Transaction action shows the user if the transaction was succesful
     *
     * @param \Ig\IgRuckzuckevent\Domain\Model\SubscriptionOffer $product
     * @param string $referer
     * @param string $transactionId
     * @param string $startDate
     *
     * @return string
     */
    public function transactionAction(\Ig\IgRuckzuckevent\Domain\Model\SubscriptionOffer $product, $referer, $transactionId, $startDate)
    {
        if ($GLOBALS['TSFE']->loginUser && $transactionId !== '') {
            $transaction = $this->braintreeService->find($transactionId);

            if ($this->isTransactionSuccessful($transaction)) {
                $transactionState = 'success';
            } else {
                $transactionState = 'error';
            }

            $this->view->assign('product', $product);
            $this->view->assign('referer', urldecode($referer));
            $this->view->assign('transaction', $transaction);
            $this->view->assign('transactionState', $transactionState);
        }
    }

    /**
     * Create a fluid template
     *
     * @param string $name
     *
     * @return \TYPO3\CMS\Fluid\View\StandaloneView
     */
    private function createFluidTemplate($name)
    {
        $extbaseFrameworkConfiguration = $this -> configurationManager -> getConfiguration(
            \TYPO3\CMS\Extbase\Configuration\ConfigurationManagerInterface::CONFIGURATION_TYPE_FRAMEWORK
        );

        $view = GeneralUtility::makeInstance(StandaloneView::class);
        $view -> setLayoutRootPaths($extbaseFrameworkConfiguration['view']['layoutRootPaths']);
        $view -> setTemplateRootPaths($extbaseFrameworkConfiguration['view']['templateRootPaths']);
        $view -> setPartialRootPaths($extbaseFrameworkConfiguration['view']['partialRootPaths']);
        $view -> setTemplate($name);

        return $view;
    }

    protected function isTransactionSuccessful($transaction)
    {
        $transactionSuccessStatuses = [
            \Braintree\Transaction::AUTHORIZED,
            \Braintree\Transaction::AUTHORIZING,
            \Braintree\Transaction::SETTLED,
            \Braintree\Transaction::SETTLING,
            \Braintree\Transaction::SETTLEMENT_CONFIRMED,
            \Braintree\Transaction::SETTLEMENT_PENDING,
            \Braintree\Transaction::SUBMITTED_FOR_SETTLEMENT
        ];

        if (in_array($transaction->status, $transactionSuccessStatuses)) {
            return true;
        }

        return false;
    }
    public function editAction(\Ig\IgRuckzuckevent\Domain\Model\Subscription $subscription)
    {
        if (!UserUtility::isAdminLoggedIn()) {
            die('permission denied');
        }
        $this->view->assign('subscription', $subscription);
    }
        public function updateAction(\Ig\IgRuckzuckevent\Domain\Model\Subscription $subscription)
    {
        if (!UserUtility::isAdminLoggedIn()) {
            die('permission denied');
        }
        $this->subscriptionRepository->update($subscription);
        $this->redirect('newSubscription', 'User', 'IgRuckzuckevent', ['user' => $subscription->getFeUser()->getUid()]);
    }

}
